from queue import Queue
import enum
import uuid
import threading
import os
import json
from typing import Dict, Any


store = {}


def save_request(request: Dict[str, Any]) -> str:
    id = str(uuid.uuid4())
    store[id] = request
    return id


def remove_request(id: str):
    try:
        del store[str(id)]
    except KeyError:
        pass


def clear_request():
    for key in store:
        del key



from django.shortcuts import render
from django.http import HttpResponse

from .globals import store, clear_request, remove_request


def index(request):
    return render(request, "sqlmiddleware/index.html", {'store': store})


def detail(request, id):
    if request.method == "DELETE":
        remove_request(id)
        return HttpResponse(status=204)
    elif request.method == "GET":
        return render(request, "sqlmiddleware/detail.html", {'data': store[str(id)]})


def clear(request):
    clear_request()
    return HttpResponse(status=204)



import inspect
import json
import os
import re
import struct
import termios
import time
import traceback
import sys
import json
from datetime import datetime
from uuid import UUID

from django.conf import settings
from django.db import connection
from django.http import HttpResponse

from .globals import save_request


class UUIDEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, UUID):
            # if the obj is uuid, we simply return the value of uuid
            return obj.hex
        return json.JSONEncoder.default(self, obj)


class TracebackLogger:
    def __init__(self):
        self.tracebacks = []

    def capture_traceback(self):
        tb = ''.join(traceback.format_stack())
        self.tracebacks.append(tb)

    def __call__(self, execute, sql, params, many, context):
        start = time.monotonic()

        try:
            result = execute(sql, params, many, context)
        except Exception as e:
            raise

        self.capture_traceback()
        return result


def get_explain_analyze(sql):
    with connection.cursor() as cursor:
        cursor.execute(f"EXPLAIN ANALYZE {sql}")
        return '\n'.join(r[0] for r in cursor.fetchall())


explain_cost_r = re.compile("\(cost=(?P<cost>[^ ]+) rows=(?P<rows>\d+) width=(?P<width>\d+)\)")


def LogSQLMiddleware(get_response):
    def middleware(request):
        logger = TracebackLogger()

        with connection.execute_wrapper(logger):
            response = get_response(request)

        queries = connection.queries.copy()

        for tb, q in zip(logger.tracebacks, queries):
            q['traceback'] = tb
            explain = get_explain_analyze(q['sql'])

            result = explain_cost_r.search(explain.split('\n')[0])
            q['explain'] = explain
            q['explain_cost'] = result.group("cost")
            q['explain_rows'] = result.group("rows")
            q['explain_width'] = result.group("width")

        total_duration = sum(float(q['time']) for q in queries)
        request_data = {
            'method': request.method,
            'created_at': datetime.now().isoformat(),
            'endpoint': request.build_absolute_uri(),
            'total_time': total_duration,
            'total_queries': len(queries),
            'paths': json.dumps(sys.path[1:]),
            'queries': queries,
        }
        if queries:
            save_request(request_data)

        return response

    return middleware



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-sql-middleware",  # Replace with your own username
    version="0.0.3",
    author="Fata Nugraha",
    author_email="fatanugraha@outlook.com",
    description="Simple SQL Middleware for Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fatanugraha/django-sql-middleware",
    install_requires=['django',],
    packages=setuptools.find_packages(exclude=["tests*"]),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from .views import index, detail, clear
from django.urls import path

app_name = 'sqlmiddleware'

urlpatterns = [
    path("", index),
    path("<uuid:id>/", detail, name="detail"),
    path("clear/", clear, name="clear"),
]
